@@ -12,12 +12,14 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import collections
 import io
 import os
 import subprocess
 
 import fixtures
 import mock
+from mistralclient.api import base as mistralclient_base
 from novaclient import exceptions
 from oslo_config import fixture as config_fixture
 from oslotest import base
@@ -39,8 +41,7 @@ class BaseTestCase(base.BaseTestCase):
 
 class TestUndercloud(BaseTestCase):
     @mock.patch('instack_undercloud.undercloud._configure_logging')
-    @mock.patch('instack_undercloud.undercloud._check_hostname')
-    @mock.patch('instack_undercloud.undercloud._check_memory')
+    @mock.patch('instack_undercloud.undercloud._validate_configuration')
     @mock.patch('instack_undercloud.undercloud._run_command')
     @mock.patch('instack_undercloud.undercloud._post_config')
     @mock.patch('instack_undercloud.undercloud._run_orc')
@@ -49,13 +50,12 @@ class TestUndercloud(BaseTestCase):
     @mock.patch('instack_undercloud.undercloud._load_config')
     def test_install(self, mock_load_config, mock_generate_environment,
                      mock_run_instack, mock_run_orc, mock_post_config,
-                     mock_run_command, mock_check_memory, mock_check_hostname,
+                     mock_run_command, mock_validate_configuration,
                      mock_configure_logging):
         fake_env = mock.MagicMock()
         mock_generate_environment.return_value = fake_env
         undercloud.install('.')
-        self.assertEqual(True, mock_check_hostname.called)
-        self.assertEqual(True, mock_check_memory.called)
+        self.assertTrue(mock_validate_configuration.called)
         mock_generate_environment.assert_called_with('.')
         mock_run_instack.assert_called_with(fake_env)
         mock_run_orc.assert_called_with(fake_env)
@@ -76,6 +76,19 @@ class TestUndercloud(BaseTestCase):
         self.assertEqual('http://bletchley:5000/v2.0',
                          undercloud._extract_from_stackrc('OS_AUTH_URL'))
 
+    @mock.patch('instack_undercloud.undercloud._check_hostname')
+    @mock.patch('instack_undercloud.undercloud._check_memory')
+    @mock.patch('instack_undercloud.undercloud._check_sysctl')
+    @mock.patch('instack_undercloud.undercloud._validate_network')
+    def test_validate_configuration(self, mock_validate_network,
+                                    mock_check_memory, mock_check_hostname,
+                                    mock_check_sysctl):
+        undercloud._validate_configuration()
+        self.assertTrue(mock_validate_network.called)
+        self.assertTrue(mock_check_memory.called)
+        self.assertTrue(mock_check_hostname.called)
+        self.assertTrue(mock_check_sysctl.called)
+
 
 class TestCheckHostname(BaseTestCase):
     @mock.patch('instack_undercloud.undercloud._run_command')
@@ -173,6 +186,18 @@ class TestCheckMemory(BaseTestCase):
         self.assertRaises(RuntimeError, undercloud._check_memory)
 
 
+class TestCheckSysctl(BaseTestCase):
+    @mock.patch('os.path.isfile')
+    def test_missing_options(self, mock_isfile):
+        mock_isfile.return_value = False
+        self.assertRaises(RuntimeError, undercloud._check_sysctl)
+
+    @mock.patch('os.path.isfile')
+    def test_available_option(self, mock_isfile):
+        mock_isfile.return_value = True
+        undercloud._check_sysctl()
+
+
 class TestGenerateEnvironment(BaseTestCase):
     def setUp(self):
         super(TestGenerateEnvironment, self).setUp()
@@ -191,6 +216,11 @@ class TestGenerateEnvironment(BaseTestCase):
         self.useFixture(self.mock_distro)
         self.mock_distro.mock.return_value = [
             'Red Hat Enterprise Linux Server 7.1']
+        stackrc_exists_patcher = mock.patch(
+            'instack_undercloud.undercloud._stackrc_exists',
+            return_value=False)
+        stackrc_exists_patcher.start()
+        self.addCleanup(stackrc_exists_patcher.stop)
 
     @mock.patch('socket.gethostname')
     def test_hostname_set(self, mock_gethostname):
@@ -254,7 +284,7 @@ class TestGenerateEnvironment(BaseTestCase):
         env = undercloud._generate_environment('.')
         endpoint_vars = {k: v for (k, v) in env.items()
                          if k.startswith('UNDERCLOUD_ENDPOINT')}
-        self.assertEqual(30, len(endpoint_vars))
+        self.assertEqual(39, len(endpoint_vars))
         # Spot check one service
         self.assertEqual('http://192.0.2.1:5000',
                          env['UNDERCLOUD_ENDPOINT_KEYSTONE_PUBLIC'])
@@ -266,7 +296,7 @@ class TestGenerateEnvironment(BaseTestCase):
         self.assertEqual('http://192.0.2.1:8080/v1/AUTH_%(tenant_id)s',
                          env['UNDERCLOUD_ENDPOINT_SWIFT_PUBLIC'])
 
-    def test_generate_endpoints_ssl(self):
+    def test_generate_endpoints_ssl_manual(self):
         conf = config_fixture.Config()
         self.useFixture(conf)
         conf.config(undercloud_service_certificate='test.pem')
@@ -282,6 +312,22 @@ class TestGenerateEnvironment(BaseTestCase):
         self.assertEqual('https://192.0.2.2:13808/v1/AUTH_%(tenant_id)s',
                          env['UNDERCLOUD_ENDPOINT_SWIFT_PUBLIC'])
 
+    def test_generate_endpoints_ssl_auto(self):
+        conf = config_fixture.Config()
+        self.useFixture(conf)
+        conf.config(generate_service_certificate=True)
+        env = undercloud._generate_environment('.')
+        # Spot check one service
+        self.assertEqual('https://192.0.2.2:13000',
+                         env['UNDERCLOUD_ENDPOINT_KEYSTONE_PUBLIC'])
+        self.assertEqual('http://192.0.2.1:5000',
+                         env['UNDERCLOUD_ENDPOINT_KEYSTONE_INTERNAL'])
+        self.assertEqual('http://192.0.2.1:35357',
+                         env['UNDERCLOUD_ENDPOINT_KEYSTONE_ADMIN'])
+        # Also check that the tenant id part is preserved
+        self.assertEqual('https://192.0.2.2:13808/v1/AUTH_%(tenant_id)s',
+                         env['UNDERCLOUD_ENDPOINT_SWIFT_PUBLIC'])
+
     def test_absolute_cert_path(self):
         conf = config_fixture.Config()
         self.useFixture(conf)
@@ -447,19 +493,29 @@ class TestConfigureSshKeys(base.BaseTestCase):
 
 class TestPostConfig(base.BaseTestCase):
     @mock.patch('novaclient.client.Client', autospec=True)
+    @mock.patch('mistralclient.api.client.client', autospec=True)
+    @mock.patch('instack_undercloud.undercloud._delete_default_flavors')
     @mock.patch('instack_undercloud.undercloud._copy_stackrc')
     @mock.patch('instack_undercloud.undercloud._get_auth_values')
     @mock.patch('instack_undercloud.undercloud._configure_ssh_keys')
     @mock.patch('instack_undercloud.undercloud._ensure_flavor')
-    def test_post_config(self, mock_ensure_flavor, mock_configure_ssh_keys,
-                         mock_get_auth_values, mock_copy_stackrc, mock_client):
+    @mock.patch('instack_undercloud.undercloud._post_config_mistral')
+    def test_post_config(self, mock_post_config_mistral, mock_ensure_flavor,
+                         mock_configure_ssh_keys, mock_get_auth_values,
+                         mock_copy_stackrc, mock_delete, mock_mistral_client,
+                         mock_nova_client):
+        instack_env = {
+            'UNDERCLOUD_ENDPOINT_MISTRAL_PUBLIC': 'http://192.0.2.1:8989/v2',
+        }
         mock_get_auth_values.return_value = ('aturing', '3nigma', 'hut8',
                                              'http://bletchley:5000/v2.0')
         mock_instance = mock.Mock()
-        mock_client.return_value = mock_instance
-        undercloud._post_config()
-        mock_client.assert_called_with(2, 'aturing', '3nigma', 'hut8',
-                                       'http://bletchley:5000/v2.0')
+        mock_nova_client.return_value = mock_instance
+        mock_instance_mistral = mock.Mock()
+        mock_mistral_client.return_value = mock_instance_mistral
+        undercloud._post_config(instack_env)
+        mock_nova_client.assert_called_with(2, 'aturing', '3nigma', 'hut8',
+                                            'http://bletchley:5000/v2.0')
         self.assertTrue(mock_copy_stackrc.called)
         mock_configure_ssh_keys.assert_called_with(mock_instance)
         calls = [mock.call(mock_instance, 'baremetal'),
@@ -470,6 +526,85 @@ class TestPostConfig(base.BaseTestCase):
                  mock.call(mock_instance, 'swift-storage', 'swift-storage'),
                  ]
         mock_ensure_flavor.assert_has_calls(calls)
+        mock_post_config_mistral.assert_called_once_with(instack_env,
+                                                         mock_instance_mistral)
+
+    def test_create_default_plan(self):
+        mock_mistral = mock.Mock()
+        mock_mistral.environments.list.return_value = []
+        mock_mistral.executions.get.return_value = mock.Mock(state="SUCCESS")
+
+        undercloud._create_default_plan(mock_mistral)
+        mock_mistral.executions.create.assert_called_once_with(
+            'tripleo.plan_management.v1.create_default_deployment_plan',
+            workflow_input={
+                'container': 'overcloud',
+                'queue_name': mock.ANY
+            }
+        )
+
+    def test_create_default_plan_existing(self):
+        mock_mistral = mock.Mock()
+        environment = collections.namedtuple('environment', ['name'])
+        mock_mistral.environments.list.return_value = [
+            environment(name='overcloud')
+        ]
+
+        undercloud._create_default_plan(mock_mistral)
+        mock_mistral.executions.create.assert_not_called()
+
+    def test_create_config_environment(self):
+        mock_mistral = mock.Mock()
+        mock_mistral.environments.get.side_effect = (
+            mistralclient_base.APIException)
+
+        env = {
+            "UNDERCLOUD_CEILOMETER_SNMPD_PASSWORD": "snmpd-pass"
+        }
+        json_string = '{"undercloud_ceilometer_snmpd_password": "snmpd-pass"}'
+
+        undercloud._create_mistral_config_environment(env, mock_mistral)
+
+        mock_mistral.environments.create.assert_called_once_with(
+            name="tripleo.undercloud-config",
+            variables=json_string)
+
+    def test_create_config_environment_existing(self):
+        mock_mistral = mock.Mock()
+        environment = collections.namedtuple('environment', ['name'])
+        mock_mistral.environments.get.return_value = environment(
+            name='overcloud')
+        env = {
+            "UNDERCLOUD_CEILOMETER_SNMPD_PASSWORD": "snmpd-pass"
+        }
+
+        undercloud._create_mistral_config_environment(env, mock_mistral)
+        mock_mistral.executions.create.assert_not_called()
+
+    def test_prepare_ssh_environment(self):
+        mock_mistral = mock.Mock()
+        undercloud._prepare_ssh_environment(mock_mistral)
+        mock_mistral.executions.create.assert_called_once_with(
+            'tripleo.validations.v1.copy_ssh_key')
+
+    @mock.patch('time.sleep')
+    def test_create_default_plan_timeout(self, mock_sleep):
+        mock_mistral = mock.Mock()
+        mock_mistral.environments.list.return_value = []
+        mock_mistral.executions.get.return_value = mock.Mock(state="RUNNING")
+
+        self.assertRaises(
+            RuntimeError,
+            undercloud._create_default_plan, mock_mistral, timeout=0)
+
+    def test_create_default_plan_failed(self):
+        mock_mistral = mock.Mock()
+        mock_mistral.environments.list.return_value = []
+        mock_mistral.executions.get.return_value = mock.Mock(state="ERROR")
+
+        self.assertRaises(
+            RuntimeError,
+            undercloud._create_default_plan, mock_mistral)
 
     @mock.patch('instack_undercloud.undercloud._run_command')
     def test_copy_stackrc(self, mock_run):
@@ -481,6 +616,53 @@ class TestPostConfig(base.BaseTestCase):
                  ]
         mock_run.assert_has_calls(calls)
 
+    @mock.patch('instack_undercloud.undercloud._stackrc_exists')
+    def test_member_role_exists_nostackrc(self, mock_stackrc_exists):
+        mock_stackrc_exists.return_value = False
+        instack_env = {}
+        undercloud._member_role_exists(instack_env)
+        self.assertEqual({'MEMBER_ROLE_EXISTS': 'False'}, instack_env)
+
+    def _mock_ksclient_roles(self, mock_auth_values, mock_ksdiscover, roles):
+        mock_auth_values.return_value = ('user', 'password',
+                                         'tenant', 'http://test:123')
+        mock_discover = mock.Mock()
+        mock_ksdiscover.return_value = mock_discover
+        mock_client = mock.Mock()
+        mock_roles = mock.Mock()
+        mock_role_list = []
+        for role in roles:
+            mock_role = mock.Mock()
+            mock_role.name = role
+            mock_role_list.append(mock_role)
+        mock_roles.list.return_value = mock_role_list
+        mock_client.roles = mock_roles
+        mock_discover.create_client.return_value = mock_client
+
+    @mock.patch('keystoneclient.discover.Discover')
+    @mock.patch('instack_undercloud.undercloud._get_auth_values')
+    @mock.patch('instack_undercloud.undercloud._stackrc_exists')
+    def test_member_role_exists(self, mock_stackrc_exists, mock_auth_values,
+                                mock_ksdiscover):
+        mock_stackrc_exists.return_value = True
+        self._mock_ksclient_roles(mock_auth_values, mock_ksdiscover,
+                                  ['admin'])
+        instack_env = {}
+        undercloud._member_role_exists(instack_env)
+        self.assertEqual({'MEMBER_ROLE_EXISTS': 'False'}, instack_env)
+
+    @mock.patch('keystoneclient.discover.Discover')
+    @mock.patch('instack_undercloud.undercloud._get_auth_values')
+    @mock.patch('instack_undercloud.undercloud._stackrc_exists')
+    def test_member_role_exists_true(self, mock_stackrc_exists,
+                                     mock_auth_values, mock_ksdiscover):
+        mock_stackrc_exists.return_value = True
+        self._mock_ksclient_roles(mock_auth_values, mock_ksdiscover,
+                                  ['admin', '_member_'])
+        instack_env = {}
+        undercloud._member_role_exists(instack_env)
+        self.assertEqual({'MEMBER_ROLE_EXISTS': 'True'}, instack_env)
+
     def _create_flavor_mocks(self):
         mock_nova = mock.Mock()
         mock_nova.flavors.create = mock.Mock()
@@ -519,3 +701,15 @@ class TestPostConfig(base.BaseTestCase):
         values = undercloud._get_auth_values()
         expected = ('aturing', '3nigma', 'hut8', 'http://bletchley:5000/v2.0')
         self.assertEqual(expected, values)
+
+    def test_delete_default_flavors(self):
+        class FakeFlavor(object):
+            def __init__(self, id_, name):
+                self.id = id_
+                self.name = name
+        mock_instance = mock.Mock()
+        mock_flavors = [FakeFlavor('f00', 'foo'),
+                        FakeFlavor('8ar', 'm1.large')]
+        mock_instance.flavors.list.return_value = mock_flavors
+        undercloud._delete_default_flavors(mock_instance)
+        mock_instance.flavors.delete.assert_called_once_with('8ar')
